@@ -42,7 +42,7 @@ class InstallCommand(Command):
                 ['-e', '--engine'],
                 {
                 'action': 'store',
-                  'choices': ['mariadb', 'mysqli', 'pgsql'],
+                  'choices': ['mariadb', 'mysqli', 'pgsql', 'sqlsrv'],
                   'default': self.C.get('defaultEngine'),
                   'help': 'database engine to use',
                   'metavar': 'engine'
@@ -88,7 +88,7 @@ class InstallCommand(Command):
         name = M.get('identifier')
         dataDir = self.Wp.getPath(name, 'data')
         if not os.path.isdir(dataDir):
-            mkdir(dataDir, 0777)
+            mkdir(dataDir, 0o777)
 
         kwargs = {
             'engine': engine,
