@@ -25,7 +25,7 @@ http://github.com/FMCorz/mdk
 import logging
 import os
 import gzip
-import urllib
+import urllib.request, urllib.parse, urllib.error
 from ..command import Command
 from ..tools import process, question
 from ..phpunit import PHPUnit
@@ -56,6 +56,14 @@ class PhpunitCommand(Command):
                 'metavar': 'testcase'
             }
         ),
+        (
+            ['-s', '--testsuite'],
+            {
+                'default': None,
+                'help': 'testsuite to run',
+                'metavar': 'testsuite'
+            }
+        ),
         (
             ['-u', '--unittest'],
             {
@@ -64,6 +72,22 @@ class PhpunitCommand(Command):
                 'metavar': 'path'
             }
         ),
+        (
+            ['-k', '--skip-init'],
+            {
+                'action': 'store_true',
+                'dest': 'skipinit',
+                'help': 'allows tests to start quicker when the instance is already initialised'
+            }
+        ),
+        (
+            ['-q', '--stop-on-failure'],
+            {
+                'action': 'store_true',
+                'dest': 'stoponfailure',
+                'help': 'stop execution upon first failure or error'
+            }
+        ),
         (
             ['-c', '--coverage'],
             {
@@ -79,6 +103,15 @@ class PhpunitCommand(Command):
                 'metavar': 'filter'
             }
         ),
+        (
+            ['--repeat'],
+            {
+                'default': None,
+                'help': 'run tests repeatedly for the given number of times',
+                'metavar': 'times',
+                'type': int
+            }
+        ),
         (
             ['name'],
             {
@@ -109,14 +142,50 @@ class PhpunitCommand(Command):
         # Create the Unit test object.
         PU = PHPUnit(self.Wp, M)
 
+        # Skip init.
+        if not args.skipinit:
+            self.init(M, PU, args)
+
+        # Automatically add the suffix _testsuite.
+        testsuite = args.testsuite
+        if testsuite and not testsuite.endswith('_testsuite'):
+            testsuite += '_testsuite'
+
+        # Check repeat arg.
+        repeat = args.repeat
+        if repeat:
+            # Make sure repeat is greater than 1.
+            if repeat <= 1:
+                repeat = None
+
+        kwargs = {
+            'coverage': args.coverage,
+            'filter': args.filter,
+            'testcase': args.testcase,
+            'testsuite': testsuite,
+            'unittest': args.unittest,
+            'stopon': [] if not args.stoponfailure else ['failure'],
+            'repeat': repeat
+        }
+
+        if args.run:
+            PU.run(**kwargs)
+            if args.coverage:
+                logging.info('Code coverage is available at: \n %s', (PU.getCoverageUrl()))
+        else:
+            logging.info('Start PHPUnit:\n %s', (' '.join(PU.getCommand(**kwargs))))
+
+    def init(self, M, PU, args):
+        """Initialises PHP Unit"""
+
         # Install Composer
         if PU.usesComposer():
             if not os.path.isfile(os.path.join(M.get('path'), 'composer.phar')):
                 logging.info('Installing Composer')
                 cliFile = 'phpunit_install_composer.php'
                 cliPath = os.path.join(M.get('path'), 'phpunit_install_composer.php')
-                (to, headers) = urllib.urlretrieve('http://getcomposer.org/installer', cliPath)
-                if headers.dict.get('content-encoding') == 'gzip':
+                (to, headers) = urllib.request.urlretrieve('http://getcomposer.org/installer', cliPath)
+                if headers.get('content-encoding') == 'gzip':
                     f = gzip.open(cliPath, 'r')
                     content = f.read()
                     f.close()
@@ -127,32 +196,14 @@ class PhpunitCommand(Command):
                 os.remove(cliPath)
                 M.cli('composer.phar', args='install --dev', stdout=None, stderr=None)
 
-        # Run cli
-        try:
+        # If Oracle, ask the user for a Behat prefix, if not set.
+        prefix = M.get('phpunit_prefix')
+        if M.get('dbtype') == 'oci' and (args.force or not prefix or len(prefix) > 2):
+            while not prefix or len(prefix) > 2:
+                prefix = question('What prefix would you like to use? (Oracle, max 2 chars)')
+        else:
+            prefix = None
 
-            # If Oracle, ask the user for a Behat prefix, if not set.
-            prefix = M.get('phpunit_prefix')
-            if M.get('dbtype') == 'oci' and (args.force or not prefix or len(prefix) > 2):
-                while not prefix or len(prefix) > 2:
-                    prefix = question('What prefix would you like to use? (Oracle, max 2 chars)')
-            else:
-                prefix = None
+        PU.init(force=args.force, prefix=prefix)
 
-            PU.init(force=args.force, prefix=prefix)
 
-            kwargs = {
-                'coverage': args.coverage,
-                'filter': args.filter,
-                'testcase': args.testcase,
-                'unittest': args.unittest
-            }
-
-            if args.run:
-                PU.run(**kwargs)
-                if args.coverage:
-                    logging.info('Code coverage is available at: \n %s', (PU.getCoverageUrl()))
-            else:
-                logging.info('Start PHPUnit:\n %s' % (' '.join(PU.getCommand(**kwargs))))
-
-        except Exception as e:
-            raise e
