@@ -23,13 +23,10 @@ http://github.com/FMCorz/mdk
 """
 
 import logging
-import MySQLdb as mysql
-import psycopg2 as pgsql
-
+from io import IOBase
 
 # TODO: Clean up the mess caused by the different engines and libraries.
 
-
 class DB(object):
 
     conn = None
@@ -43,6 +40,7 @@ class DB(object):
         self.options = options
 
         if engine in ('mysqli', 'mariadb'):
+            import MySQLdb as mysql
 
             if 'fuckfred' in options['passwd']:
                 raise Exception('Could not establish connexion with MySQL, bad language used!')
@@ -57,7 +55,8 @@ class DB(object):
             self.cur = self.conn.cursor()
 
         elif engine == 'pgsql':
-            # psycopg2.
+            import psycopg2 as pgsql
+
             self.conn = pgsql.connect(
                 host=str(options['host']),
                 port=int(options['port']),
@@ -69,6 +68,29 @@ class DB(object):
             except:
                 raise Exception('Connexion failed! Make sure the database \'%s\' exists.' % str(options['user']))
 
+        elif engine == 'sqlsrv':
+            import pyodbc
+
+            host = str(options['host'])
+            port = int(options['port'])
+            user = str(options['user'])
+            password = str(options['passwd'])
+
+            # Look for installed ODBC Driver for SQL Server.
+            drivers = pyodbc.drivers()
+            sqlsrvdriver = next((driver for driver in drivers if "for SQL Server" in driver), None)
+            if sqlsrvdriver is None:
+                installurl = 'https://sqlchoice.azurewebsites.net/en-us/sql-server/developer-get-started/python'
+                raise Exception("You need to install an ODBC Driver for SQL Server. Check out %s for more info." % installurl)
+
+            logging.debug('Using %s' % sqlsrvdriver)
+
+            connectionstr = "DRIVER=%s;SERVER=%s;PORT=%d;UID=%s;PWD=%s" \
+                            % (sqlsrvdriver, host, port, user, password)
+            self.conn = pyodbc.connect(connectionstr)
+            self.conn.autocommit = True
+            self.cur = self.conn.cursor()
+
         else:
             raise Exception('DB engine %s not supported' % engine)
 
@@ -99,9 +121,20 @@ class DB(object):
             pass
 
         if self.engine in ('mysqli', 'mariadb'):
-            sql = 'CREATE DATABASE `%s` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci' % db
+
+            if 'charset' in self.options and self.options['charset'] == 'utf8mb4':
+                placeholders = (db, 'utf8mb4', 'utf8mb4_unicode_ci')
+            else:
+                placeholders = (db, 'utf8', 'utf8_unicode_ci')
+
+            sql = 'CREATE DATABASE `%s` CHARACTER SET %s COLLATE %s' % placeholders
         elif self.engine == 'pgsql':
             sql = 'CREATE DATABASE "%s" WITH ENCODING \'UNICODE\'' % db
+        elif self.engine == 'sqlsrv':
+            sql = 'CREATE DATABASE "%s" COLLATE Latin1_General_CS_AS;' \
+                  'ALTER DATABASE "%s" SET ANSI_NULLS ON;' \
+                  'ALTER DATABASE "%s" SET QUOTED_IDENTIFIER ON;' \
+                  'ALTER DATABASE "%s" SET READ_COMMITTED_SNAPSHOT ON;' % (db, db, db, db)
 
         logging.debug(sql)
         self.cur.execute(sql)
@@ -120,6 +153,9 @@ class DB(object):
         elif self.engine == 'pgsql':
             sql = "SELECT COUNT('*') FROM pg_database WHERE datname='%s'" % db
 
+        elif self.engine == 'sqlsrv':
+            sql = "SELECT COUNT('*') FROM master.dbo.sysdatabases WHERE name = '%s'" % db
+
         logging.debug(sql)
         self.cur.execute(sql)
         count = self.cur.fetchone()[0]
@@ -137,7 +173,7 @@ class DB(object):
 
         if self.engine in ('mysqli', 'mariadb'):
             sql = 'DROP DATABASE `%s`' % db
-        elif self.engine == 'pgsql':
+        elif self.engine in ('pgsql', 'sqlsrv'):
             sql = 'DROP DATABASE "%s"' % db
 
         logging.debug(sql)
@@ -153,7 +189,7 @@ class DB(object):
 
         if self.engine not in ('mysqli', 'mariadb'):
             raise Exception('Function dump not supported by %s' % self.engine)
-        if not type(fd) == file:
+        if not isinstance(fd, IOBase):
             raise Exception('Passed parameter is not a file object')
 
         # Looping over selected tables
@@ -192,6 +228,8 @@ class DB(object):
             self.cur.execute('USE %s' % db)
 
         elif self.engine == 'pgsql':
+            import psycopg2 as pgsql
+
             if self.cur:
                 self.cur.close()
             if self.conn:
