@@ -26,6 +26,8 @@ import os
 import re
 import logging
 import shutil
+import subprocess
+import json
 from tempfile import gettempdir
 
 from .tools import getMDLFromCommitMessage, mkdir, process, parseBranch
@@ -85,6 +87,8 @@ class Moodle(object):
 
         if type(value) == bool:
             value = 'true' if value else 'false'
+        elif type(value) in (dict, list):
+            value = "json_decode('" + json.dumps(value) + "', true)"
         elif type(value) != int:
             value = "'" + str(value) + "'"
         value = str(value)
@@ -254,7 +258,7 @@ class Moodle(object):
         try:
             # Trying to smart guess the last commit needed
             if smartSearch:
-                commits = self.git().log(since=branch, count=C.get('smartHeadCommitLimit'), format='%s_____%h').split('\n')[:-1]
+                commits = self.git().log(since=branch, count=C.get('smartHeadCommitLimit'), format='%s_____%H').split('\n')[:-1]
 
                 # Looping over the last commits to find the commit messages that match the MDL-12345.
                 candidate = None
@@ -273,7 +277,7 @@ class Moodle(object):
             if not headcommit:
                 upstreamremote = C.get('upstreamRemote')
                 stablebranch = self.get('stablebranch')
-                headcommit = self.git().hashes(ref='%s/%s' % (upstreamremote, stablebranch), limit=1, format='%h')[0]
+                headcommit = self.git().hashes(ref='%s/%s' % (upstreamremote, stablebranch), limit=1, format='%H')[0]
 
         except GitException:
             logging.warning('Could not resolve the head commit')
@@ -308,7 +312,7 @@ class Moodle(object):
         currentFailDumpPath = self.get('behat_faildump_path')
         if faildumppath and currentFailDumpPath != faildumppath:
             self.updateConfig('behat_faildump_path', faildumppath)
-        else:
+        elif (not faildumppath and currentFailDumpPath):
             self.removeConfig('behat_faildump_path')
 
         if not currentPrefix or force:
@@ -363,9 +367,9 @@ class Moodle(object):
             'installed': self.isInstalled(),
             'identifier': self.identifier
         }
-        for (k, v) in self.config.items():
+        for (k, v) in list(self.config.items()):
             info[k] = v
-        for (k, v) in self.version.items():
+        for (k, v) in list(self.version.items()):
             info[k] = v
         return info
 
@@ -391,8 +395,12 @@ class Moodle(object):
             fullname = self.identifier.replace('-', ' ').replace('_', ' ').title()
             fullname = fullname + ' ' + C.get('wording.%s' % engine)
 
+        dboptions =  C.get('db.%s' % engine)
+        if engine in ('mysqli', 'mariadb') and self.branch_compare(31):
+            dboptions['charset'] = 'utf8mb4'
+
         logging.info('Creating database...')
-        db = DB(engine, C.get('db.%s' % engine))
+        db = DB(engine, dboptions)
         if db.dbexists(dbname):
             if dropDb:
                 db.dropdb(dbname)
@@ -405,14 +413,14 @@ class Moodle(object):
 
         logging.info('Installing %s...' % self.identifier)
         cli = 'admin/cli/install.php'
-        params = (wwwroot, dataDir, engine, dbname, C.get('db.%s.user' % engine), C.get('db.%s.passwd' % engine), C.get('db.%s.host' % engine), fullname, self.identifier, C.get('login'), C.get('passwd'))
-        args = '--wwwroot="%s" --dataroot="%s" --dbtype="%s" --dbname="%s" --dbuser="%s" --dbpass="%s" --dbhost="%s" --fullname="%s" --shortname="%s" --adminuser="%s" --adminpass="%s" --allow-unstable --agree-license --non-interactive' % params
+        params = (wwwroot, dataDir, engine, dbname, C.get('db.%s.user' % engine), C.get('db.%s.passwd' % engine), C.get('db.%s.host' % engine), C.get('db.%s.port' % engine), C.get('db.tablePrefix'), fullname, self.identifier, C.get('login'), C.get('passwd'))
+        args = '--wwwroot="%s" --dataroot="%s" --dbtype="%s" --dbname="%s" --dbuser="%s" --dbpass="%s" --dbhost="%s" --dbport="%s" --prefix="%s" --fullname="%s" --shortname="%s" --adminuser="%s" --adminpass="%s" --allow-unstable --agree-license --non-interactive' % params
         result = self.cli(cli, args, stdout=None, stderr=None)
         if result[0] != 0:
             raise InstallException('Error while running the install, please manually fix the problem.\n- Command was: %s %s %s' % (C.get('php'), cli, args))
 
         configFile = os.path.join(self.path, 'config.php')
-        os.chmod(configFile, 0666)
+        os.chmod(configFile, 0o666)
         try:
             if C.get('path') != '' and C.get('path') != None:
                 self.addConfig('sessioncookiepath', '/%s/%s/' % (C.get('path'), self.identifier))
@@ -424,7 +432,7 @@ class Moodle(object):
         # Add forced $CFG to the config.php if some are globally defined.
         forceCfg = C.get('forceCfg')
         if isinstance(forceCfg, dict):
-            for cfgKey, cfgValue in forceCfg.iteritems():
+            for cfgKey, cfgValue in forceCfg.items():
                 try:
                     logging.info('Setting up forced $CFG->%s to \'%s\' in config.php', cfgKey, cfgValue)
                     self.addConfig(cfgKey, cfgValue)
@@ -710,7 +718,7 @@ class Moodle(object):
         if os.path.isdir(dataroot):
             logging.debug('Deleting dataroot content (%s)' % (dataroot))
             shutil.rmtree(dataroot)
-            mkdir(dataroot, 0777)
+            mkdir(dataroot, 0o777)
 
         # Drop the database
         dbname = self.get('dbname')
@@ -749,7 +757,7 @@ class Moodle(object):
         logging.info('Searching for the head commit...')
         if ref:
             try:
-                headcommit = self.git().hashes(ref=ref, limit=1, format='%h')[0]
+                headcommit = self.git().hashes(ref=ref, limit=1, format='%H')[0]
             except GitException:
                 logging.warning('Could not resolve a head commit using the reference: %s' % (ref))
                 headcommit = None
@@ -763,6 +771,7 @@ class Moodle(object):
             logging.error('Head commit not resolved, aborting update of tracker fields')
             return False
 
+        headcommit = headcommit[:10]
         logging.debug('Head commit resolved to %s' % (headcommit))
 
         J = Jira()
@@ -801,3 +810,25 @@ class Moodle(object):
         # Return to previous branch
         if not nocheckout:
             self.checkout_stable(False)
+
+    def uninstallPlugins(self, name):
+        """Calls the CLI to uninstall a plugin"""
+
+        if not self.branch_compare(37):
+            raise Exception('Uninstalling plugins is only available from Moodle 3.7.')
+
+        cli = '/admin/cli/uninstall_plugins.php'
+        args = '--plugins=' + name + ' --run'
+
+        result = self.cli(cli, args, stdout=subprocess.PIPE, stderr=None)
+        try:
+            resultstring = result[1].split('\n', 1)
+            cannotuninstall = resultstring[0].rfind('Can not be uninstalled')
+            if cannotuninstall != -1:
+                raise Exception('The plugin could not be uninstalled: %s' % result[1])
+        except IndexError as e:
+            # We should always have some text returned and so should not end up here. And we
+            # raise the exception as we're unsure whether the plugin was uninstalled properly
+            # so it's better to halt the process.
+            logging.error('The plugin uninstall cli code has changed and I need to be updated.')
+            raise e
